DAT  fileName byte "RgbLed110729f", 0
CON{{ 

  Modified from RedGreenBlue110729f

  ---- Private Notes -------
  110728a Move LED control to child object (this object).
  29c the flashing doesn't seem to work correctly.
  29d works a little.
  29e only flashes once.
  29f works okay.
  Change name from RedGreenBlue110729f
  to RgbLed110729f
  110729f start changing to child only object.
  
  ---- Public Notes -------
  
  Use to drive a common cathode RGB LED.
  
  The "Init" method needs to be called before any
  other methods.
  
  
}} 

  ' LED command enumeration
  #0, _OffLed, _StartSolidLed, _StartFlashLed, _SolidLed, _FlashLed, _StartDriver
                         
  _DriverOff = $0F              ' used to indicate a cog has not been started.
                         
VAR

  long cog
  long ledStack[64]
  long cycleCount
  
  byte command
  byte red, green, blue 

PUB Init(localRedPin, localGreenPin, localBluePin)
'' Call Init method to assign pin numbers.
'' This needs to be called before calling other methods.

  redMask := 1 << localRedPin
  greenMask := 1 << localGreenPin
  blueMask := 1 << localBluePin
  
  commandPtr := @command
  redPtr := @red
  greenPtr := @green
  bluePtr := @blue

  onTimePtr := @onTime
  cycleCountPtr := @cycleCount
  
  command := _OffLed 
  cog := _DriverOff
  result := commandPtr
'' commandPtr is returned so parent object
'' can access it.
  
PUB FlashLed(redByte, greenByte, blueByte, localFlashPeriod, localCycleCount)              
'' redByte how much of the on time should the red LED be on.
'' greenByte how much of the on time should the green LED be on.
'' blueByte how much of the on time should the blue LED be on.
'' All three color numbers should be between zero and 255.
'' flashTime period of a flash cycle in milliseconds.
'' The LED will be on half of the flash cycle.
'' localCycleCount is how many times to flash.
'' if localCycleCount is zero then the LED will continue to flash
'' until a different instruction is given.
'' If the driving cog has not been started, this method will start the cog.

  red := redByte
  green := greenByte
  blue :=  blueByte
  cycleCount := localCycleCount
  
  onTime := (clkfreq / 1000) * localFlashPeriod
  command := _StartFlashLed  

  if cog == _DriverOff
    cog := Start
    if cog == -1
      cog := _DriverOff
  
  if cog <> _DriverOff
    repeat while command == _StartFlashLed 

  result := cog

PUB SolidLed(redByte, greenByte, blueByte, localOnTime) | localIndex, localTime, {
            } changeTime, largeLoopStartTime, temp[6]                                       
'' redByte how much of the on time should the red LED be on.
'' greenByte how much of the on time should the green LED be on.
'' blueByte how much of the on time should the blue LED be on.
'' All three color numbers should be between zero and 255.
'' onTime time LED is on in milliseconds.
'' If onTime is set to zero then the LED will stay on
'' until a different instruction is given.
'' If the driving cog has not been started, this method will start the cog.
 
  red := redByte
  green := greenByte
  blue :=  blueByte
  
  onTime := (clkfreq / 1000) * localOnTime
  command := _StartSolidLed  

  if cog == _DriverOff
    cog := Start
    if cog == -1
      cog := _DriverOff
      
  if cog <> _DriverOff
    repeat while command == _StartSolidLed   

  result := cog

PUB ChangeColor(redByte, greenByte, blueByte)                                      
'' redByte how much of the on time should the red LED be on.
'' greenByte how much of the on time should the green LED be on.
'' blueByte how much of the on time should the blue LED be on.
'' All three color numbers should be between zero and 255.

  red := redByte
  green := greenByte
  blue :=  blueByte

  result := cog

PUB Off
'' This will turn the LED off if it is on.
'' This does not stop the cog.

  command := _OffLed
  result := cog
  
PUB Start

  command := _StartDriver
  
  if cog == _DriverOff
    cog := cognew(@enter, @command)

  if cog => 0 and cog =< 7
    repeat while command
  
  result := cog
        
PUB Stop

  result := cog
  command := _OffLed 
  if cog <> _DriverOff
    cogstop(cog)
    cog := _DriverOff

DAT

                        org

enter                   andn    outa, redMask               
                        or      dira, redMask               
                        andn    outa, greenMask        
                        or      dira, greenMask                
                        andn    outa, blueMask             
                        or      dira, blueMask
             
preloop                 wrbyte  zeroC, commandPtr             
loop                    rdbyte  previousCmd, commandPtr            
                        tjz     previousCmd, #loop
                                           
                        mov     jumpMode, previousCmd     
                        add     jumpMode, #JumpTable            
                        jmp     jumpMode   ' indirect jump


JumpTable               jmp     #loop           ' I don't think this get used 
                        jmp     #startSolidLedMethod
                        jmp     #startFlashLedMethod
                        jmp     #loop           
                        jmp     #loop           
                        jmp     #preloop        ' I don't think this get used 

DAT '----------------------- startSolidLedMethod -----------------------------------------------------------------------------

startSolidLedMethod     rdlong  onTime, onTimePtr
                        mov     startTime, cnt
                        wrbyte  solidLedC, commandPtr
                        mov     previousCmd, solidLedC
solidLedloop            rdbyte  tRed, redPtr
                        mov     countDown, #$100
                        rdbyte  tGreen, greenPtr
                        mov     brightnessTarget, countDown                                                
                        rdbyte  tBlue, bluePtr
                                                             
              
              
:localLoop              sub     brightnessTarget, #1
                        cmp     brightnessTarget, tRed wc
                        muxc    outa, redMask      
                        cmp     brightnessTarget, tGreen wc
                        muxc    outa, greenMask      
                        cmp     brightnessTarget, tBlue wc
                        muxc    outa, blueMask
                        nop     ' lots of nops so reading the color bytes     
                        nop     ' don't take up so much time compared to     
                        nop     ' the time "localLoop" takes.     
                        nop     ' This could be shortened with a loop     
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        djnz    countDown, #:localLoop
                        rdbyte  currentCmd, commandPtr  '' check for interuption
                        cmp     previousCmd, currentCmd wz '' if a new command has been
              if_nz     jmp     #exitSolidLed              '' issued then exit
                        mov     tempTime, cnt
                        sub     tempTime, startTime        
                        cmp     onTime, zeroC wz           '' Stay on mode?         
                        cmp     tempTime, onTime wc        '' Is time up?
              if_c_or_z jmp     #solidLedloop
exitSolidLed            andn    outa, redMask              '' Turn off LED
                        andn    outa, greenMask         
                        andn    outa, blueMask         
                        jmp     #loop
              
DAT '----------------------- startFlashLedMethod -----------------------------------------------------------------------------


startFlashLedMethod     rdlong  onTime, onTimePtr
                        mov     halfCycleTime, onTime
                        rdlong  cycleCountCog, cycleCountPtr
                        mov     stayOnFlag, cycleCountCog                  
                        wrbyte  flashLedC, commandPtr
                        mov     previousCmd, flashLedC
                        shr     halfCycleTime, #1       ' divide by two
outerFlashLedloop       mov     startTime, cnt
innerFlashLedLoop       rdbyte  tRed, redPtr
                        mov     countDown, #$100
                        rdbyte  tGreen, greenPtr
                        mov     brightnessTarget, countDown                                                
                        rdbyte  tBlue, bluePtr
                                                             
              
              
:localLoop              sub     brightnessTarget, #1
                        cmp     brightnessTarget, tRed wc
                        muxc    outa, redMask      
                        cmp     brightnessTarget, tGreen wc
                        muxc    outa, greenMask      
                        cmp     brightnessTarget, tBlue wc
                        muxc    outa, blueMask
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        nop          
                        djnz    countDown, #:localLoop
                        rdbyte  currentCmd, commandPtr  '' check for interuption
                        cmp     previousCmd, currentCmd wz '' if a new command has been
              if_nz     jmp     #exitFlashLed              '' issued then exit
                        mov     tempTime, cnt
                        sub     tempTime, startTime        
                        'cmp     onTime, zeroC wz           '' Stay on mode?         
                        cmp     tempTime, halfCycleTime wc        '' Is time up?
              if_c      jmp     #innerflashLedloop

                        andn    outa, redMask              '' Turn off LED
                        andn    outa, greenMask         
                        andn    outa, blueMask

ledFlashOff             rdbyte  currentCmd, commandPtr  '' check for interuption
                        cmp     previousCmd, currentCmd wz '' if a new command has been
              if_nz     jmp     #exitFlashLed              '' issued then exit
                        mov     tempTime, cnt
                        sub     tempTime, startTime        
                        cmp     tempTime, onTime wc        '' Is time up?
              if_c      jmp     #ledFlashOff            '' keep LED off until times up.
                        djnz    cycleCountCog, #outerFlashLedloop                  
                        tjz     stayOnFlag, #outerFlashLedloop
                                             
exitFlashLed            andn    outa, redMask              '' Turn off LED
                        andn    outa, greenMask         
                        andn    outa, blueMask         
                        jmp     #loop
               
DAT '----------------------- cog variables -----------------------------------------------------------------------------

zeroC                   long    0                       ''constants
offLedC                 long    _OffLed
startSolidLedC          long    _StartSolidLed
startFlashLedC          long    _StartFlashLed
solidLedC               long    _SolidLed
flashLedC               long    _FlashLed
startDriverC            long    _StartDriver

                                              
previousCmd             long    0                        
currentCmd              long    0
tRed                    long    0                       
tGreen                  long    0                       
tBlue                   long    0                       
brightnessTarget        long    0

redMask                 long    0
greenMask               long    0
blueMask                long    0
redPtr                  long    0
greenPtr                long    0
bluePtr                 long    0
commandPtr              long    0
onTime                  long    0
onTimePtr               long    0
cycleCountPtr           long    0
startTime               res     1
tempTime                res     1
halfCycleTime           res     1
jumpMode                res     1
countDown               res     1
cycleCountCog           res     1
stayOnFlag              res     1

                        fit

                        